import '../../flutter_dynamic_widget.dart';
import '../base.dart';
import './index.dart';

class MaterialBase extends WidgetLibrary {
  MaterialBase(this.data, this.widgetContext);

  final Map<String, dynamic> data;
  final WidgetContext widgetContext;

  @override
  Map<String, WidgetConfig> get library => {
        ...basics,
        ...layout,
        ...misc,
        ...themes,
        ...properties,
        'FloatingActionButton': FloatingActionButtonBase(data, widgetContext),
        'Wrap': WrapBase(data, widgetContext),
        'SizedBox': SizedBoxBase(data, widgetContext),
        'FractionallySizedBox': FractionallySizedBoxBase(data, widgetContext),
        'IconButton': IconButtonBase(data, widgetContext),
        'SingleChildScrollView': SingleChildScrollViewBase(data, widgetContext),
        "Theme": ThemeBase(data, widgetContext),
        "Form": FormBase(data, widgetContext),
      };

  @override
  WidgetConfig get base {
    if (data == null) return null;
    final name = data['name'].toString();
    final _materialBase = library[name];
    if (_materialBase != null) return _materialBase;
    return DynamicWidget(
      data: data,
      widgetContext: widgetContext,
    )?.base;
  }

  static ActionCallback onAction = (context, val) {
    DynamicWidget.onAction(context, val);
  };

  Map<String, Map<String, WidgetConfig>> get categories => {
        'Basics': basics,
        'Layout': layout,
        'Misc': misc,
      };

  Map<String, WidgetConfig> get layout => {
        'Center': CenterBase(data, widgetContext),
        'Expanded': ExpandedBase(data, widgetContext),
        'Flexible': FlexibleBase(data, widgetContext),
        'Stack': StackBase(data, widgetContext),
        'Positioned': PositionedBase(data, widgetContext),
        'Flex': FlexBase(data, widgetContext),
        'Align': AlignBase(data, widgetContext),
        'AspectRatio': AspectRatioBase(data, widgetContext),
        'Baseline': BaselineBase(data, widgetContext),
        'ConstrainedBox': ConstrainedBoxBase(data, widgetContext),
        'FittedBox': FittedBoxBase(data, widgetContext),
        'IntrinsicWidth': IntrinsicWidthBase(data, widgetContext),
        'IntrinsicHeight': IntrinsicHeightBase(data, widgetContext),
        'LimitedBox': LimitedBoxBase(data, widgetContext),
        'Offstage': OffstageBase(data, widgetContext),
        'OverflowBox': OverflowBoxBase(data, widgetContext),
        'SizedBox': SizedBoxBase(data, widgetContext),
        'Padding': PaddingBase(data, widgetContext),
        'SizedOverflowBox': SizedOverflowBoxBase(data, widgetContext),
        'Transform': TransformBase(data, widgetContext),
        'PreferredSize': PreferredSizeBase(data, widgetContext),
      };

  Map<String, WidgetConfig> get properties => {
        'IconData': IconDataBase(data, widgetContext),
        'TextStyle': TextStyleBase(data, widgetContext),
        'BoxConstraints': BoxConstraintsBase(data, widgetContext),
        'BottomNavigationBarItem':
            BottomNavigationBarItemBase(data, widgetContext),
      };

  Map<String, WidgetConfig> get themes => {
        'ThemeData': ThemeDataBase(data, widgetContext),
        'IconThemeData': IconThemeDataBase(data, widgetContext),
        'TextTheme': TextThemeBase(data, widgetContext),
        'AppBarTheme': AppBarThemeBase(data, widgetContext),
        'FloatingActionButtonThemeData':
            FloatingActionButtonThemeDataBase(data, widgetContext),
      };

  Map<String, WidgetConfig> get basics => {
        'AppBar': AppBarBase(data, widgetContext),
        'Column': ColumnBase(data, widgetContext),
        'Container': ContainerBase(data, widgetContext),
        "FlutterLogo": FlutterLogoBase(data, widgetContext),
        'Icon': IconBase(data, widgetContext),
        'Image.network': ImageNetworkBase(data, widgetContext),
        "Placeholder": PlaceholderBase(data, widgetContext),
        'RaisedButton': RaisedButtonBase(data, widgetContext),
        'RaisedButton.icon': RaisedButtonIconBase(data, widgetContext),
        "FlatButton": FlatButtonBase(data, widgetContext),
        "FlatButton.icon": FlatButtonIconBase(data, widgetContext),
        "OutlineButton": OutlineButtonBase(data, widgetContext),
        'Row': RowBase(data, widgetContext),
        'Scaffold': ScaffoldBase(data, widgetContext),
        'Text': TextBase(data, widgetContext),
      };

  Map<String, WidgetConfig> get misc => {
        'BottomNavigationBar': BottomNavigationBarBase(data, widgetContext),
        'Drawer': DrawerBase(data, widgetContext),
        'MaterialApp': MaterialAppBase(data, widgetContext),
        'WidgetsApp': WidgetsAppBase(data, widgetContext),
        'ButtonBar': ButtonBarBase(data, widgetContext),
        'Card': CardBase(data, widgetContext),
        'Chip': ChipBase(data, widgetContext),
        'Tooltip': TooltipBase(data, widgetContext),
        'Divider': DividerBase(data, widgetContext),
        'ListTile': ListTileBase(data, widgetContext),
        'CircularProgressIndicator':
            CircularProgressIndicatorBase(data, widgetContext),
        'LinearProgressIndicator':
            LinearProgressIndicatorBase(data, widgetContext),
      };
}
